@@ -1,43 +1,76 @@
 #!/usr/bin/env python3
+# -*- coding: utf-8 -*-
 from __future__ import print_function
 import os
 import json
 import shutil
 import glob
 import logging
+import signal
+import psutil  # type: ignore
 from typing import List
 from hashlib import blake2b
-from signal import signal, SIGPIPE, SIGINT, SIG_DFL
-signal(SIGPIPE, SIG_DFL)
-signal(SIGINT, SIG_DFL)
 
 SUFFIXES = ['.nsq', '.nin', '.nhr', '.nto', '.not', '.ndb', '.ntf']
 
 
-def remove_intermediates(runid: str, intermediates: List[str], protect: bool)\
-        -> None:
+def siginthandler(sig, frame):
+    print('\033[?25h')
+    end_program(2)
+
+
+def worker_init(parent_id):
+    logger = logging.getLogger(__name__)
+
+    def sig_int(signal_num, frame):
+        parent = psutil.Process(parent_id)
+        pid = os.getpid()
+        for child in parent.children():
+            if child.pid != pid:
+                if child.is_running():
+                    logger.debug(f'killing child: {child.pid}')
+                    child.kill()
+        logger.debug(f'killing parent: {parent_id}')
+        if parent.is_running():
+            parent.kill()
+        # logger.debug(f'suicide: {os.getpid()}')
+        end_program(2)
+        # psutil.Process(os.getpid()).kill()
+
+    signal.signal(signal.SIGINT, sig_int)
+
+
+def remove_intermediates(runid: str, intermediates: List[str], protect: bool) -> None:
     logger = logging.getLogger(__name__)
     if protect:
         msg = 'One or more intermediates of {} type need to be removed'
         logger.info(msg.format(','.join(intermediates)))
-        msg = 'Delete the "protect" flag in the config.json file or restart '\
+        msg = (
+            'Delete the "protect" flag in the config.json file or restart '
             'your analysis with another runid to continue.'
+        )
         logger.info(msg)
         end_program(1)
 
-    genome_tmpfiles = ['blast_results.tsv', 'keepsidx.json',
-                       'single_copy.json', 'blast_filtered.tsv',
-                       'expected_filt.json']
+    genome_tmpfiles = [
+        'blast_results.tsv',
+        'keepsidx.json',
+        'single_copy.json',
+        'blast_filtered.tsv',
+        'expected_filt.json',
+    ]
     if 'genome' in intermediates:
         for deldir in ['unaligned', 'aligned']:
             if os.path.exists(deldir):
                 shutil.rmtree(deldir)
-        for fullname in [os.path.join('.autoMLSA', fn) for
-                         fn in genome_tmpfiles]:
+        for fullname in [os.path.join('.autoMLSA', fn) for fn in genome_tmpfiles]:
             if os.path.exists(fullname):
                 os.remove(fullname)
     for delfile in glob.iglob(runid + '.nex*'):
-        os.remove(delfile)
+        logger.info('Backing up nexus and tree files to backup dir.')
+        shutil.move(delfile, 'backup')
+        if os.path.exists(delfile):
+            os.remove(delfile)
 
 
 def end_program(code: int) -> None:
@@ -51,6 +84,12 @@ def end_program(code: int) -> None:
     elif code == 0:
         msg = 'Program was a success! Congratulations!'
         logger.info(msg)
+    elif code == 2:
+        print('\033[?25h')
+        msg = 'Program was interrupted using the keyboard.'
+        logger.info(msg)
+        msg = 'Partial files may exist. Re-running the analysis is suggested.'
+        logger.info(msg)
     else:
         msg = 'Program exiting with code ({}) indicating failure.'
         logger.info(msg.format(code))
