@@ -1,4 +1,5 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
+# -*- coding: utf-8 -*-
 import os
 import argparse
 import logging
@@ -12,9 +13,10 @@ from .validate_requirements import (
     validate_requirements,
     install_blast,
     install_mafft,
-    install_iqtree
+    install_iqtree,
 )
 from .__version__ import __version__
+from ._settings import MAFFT, IQTREE
 
 
 class InstallDeps(argparse.Action):
@@ -22,7 +24,6 @@ class InstallDeps(argparse.Action):
         super().__init__(nargs=nargs, **kw)
 
     def __call__(self, parser, namespace, values, option_string=None):
-    # def __call__(self, parser, values):
         if values:
             external = values
         else:
@@ -54,8 +55,10 @@ class InstallDeps(argparse.Action):
             install = True
         validate_requirements(external)
         if install:
-            msg = 'Please provide given path {} as the --external flag to '\
+            msg = (
+                'Please provide given path {} as the --external flag to '
                 'use the installed programs.'
+            )
             logger.info(msg.format(external))
             msg = '~/.local/external will be found automatically.'
             logger.info(msg)
@@ -76,6 +79,7 @@ def config_rundir(runid: str) -> str:
         if not os.path.exists(rundir):
             try:
                 os.mkdir(rundir)
+                os.mkdir(os.path.join(rundir, 'backup'))
                 os.mkdir(os.path.join(rundir, '.autoMLSA'))
                 os.mkdir(os.path.join(rundir, '.autoMLSA', 'backups'))
                 os.mkdir(os.path.join(rundir, '.autoMLSA', 'checkpoint'))
@@ -113,8 +117,7 @@ def get_fasta_files(x: str) -> str:
         raise argparse.ArgumentTypeError("{0} is not a FASTA file".format(x))
 
 
-def init_logger(debug: bool, quiet: bool, rundir: str, runid: str) -> \
-        None:
+def init_logger(debug: bool, quiet: bool, rundir: str, runid: str) -> None:
     """Sets up logging system to file and stderr
 
     input  - parsed arguments
@@ -126,6 +129,8 @@ def init_logger(debug: bool, quiet: bool, rundir: str, runid: str) -> \
     logger.setLevel(logging.DEBUG)
 
     stderr_handler = RichHandler(rich_tracebacks=True)
+    if not sys.stderr.isatty():
+        stderr_handler = RichHandler(rich_tracebacks=True, console=Console(width=119))
     if debug:
         stderr_handler.setLevel(logging.DEBUG)
     elif quiet:
@@ -137,88 +142,120 @@ def init_logger(debug: bool, quiet: bool, rundir: str, runid: str) -> \
     # Print to file as well, if provided.
     if rundir and runid:
         logfile = os.path.join(rundir, '{}.log'.format(runid))
-        file_handler = RichHandler(console=Console(file=open(logfile, 'a')))
+        file_handler = RichHandler(console=Console(file=open(logfile, 'a'), width=119))
         logger.addHandler(file_handler)
 
 
 def run_argparse() -> argparse.Namespace:
     parser = argparse.ArgumentParser(
         description='This is a rewrite of autoMLSA.pl. Generates automated '
-                    'multi-locus sequence analyses.',
+        'multi-locus sequence analyses.',
         epilog='Quick usage: automlsa2 --dir genomedir --query queries.fas '
-               '-t # -- runID')
+        '-t # -- runID',
+    )
+    parser.add_argument('runid', help='Name of the run directory.', type=str)
     parser.add_argument(
-        'runid', help='Name of the run directory.', type=str)
+        '--query', help='Path to file with input seq(s).', type=extant_file, nargs='+'
+    )
     parser.add_argument(
-        '--query', help='Path to file with input seq(s).', type=extant_file,
-        nargs='+')
+        '--files',
+        help='Path to the target genome FASTA files.',
+        type=get_fasta_files,
+        nargs='+',
+    )
     parser.add_argument(
-        '--files', help='Path to the target genome FASTA files.',
-        type=get_fasta_files, nargs='+')
+        '--dir',
+        help='Path to the target genome directory with FASTA files.',
+        type=extant_file,
+        nargs='+',
+    )
     parser.add_argument(
-        '--dir', help='Path to the target genome directory with FASTA files.',
-        type=extant_file, nargs='+')
+        '-e', '--evalue', help='E-value cutoff for BLAST searches. [1e-5]', type=float
+    )
     parser.add_argument(
-        '-e', '--evalue', help='E-value cutoff for BLAST searches. [1e-5]',
-        type=float)
+        '-c', '--coverage', help='Sets the coverage cut-off threshold. [50]', type=int
+    )
     parser.add_argument(
-        '-c', '--coverage', help='Sets the coverage cut-off threshold. [50]',
-        type=int)
+        '-i', '--identity', help='Sets the identity cut-off threshold. [30]', type=int
+    )
     parser.add_argument(
-        '-i', '--identity', help='Sets the identity cut-off threshold. [30]',
-        type=int)
+        '-p',
+        '--program',
+        help='Which BLAST program to run. [tblastn]',
+        choices=['blastn', 'tblastn'],
+        type=str,
+    )
     parser.add_argument(
-        '-p', '--program', help='Which BLAST program to run. [tblastn]',
-        choices=['blastn', 'tblastn'], type=str)
+        '--config', help='Path to configuration json file to copy.', type=extant_file
+    )
     parser.add_argument(
-        '--config', help='Path to configuration json file to copy.',
-        type=extant_file)
+        '--missing_check',
+        help='Use this to confirm that settings have been '
+        'checked when genes are missing.',
+        action='store_true',
+        default=False,
+    )
     parser.add_argument(
-        '--missing_check', help='Use this to confirm that settings have been '
-                                'checked when genes are missing.',
-        action='store_true', default=False)
+        '-t', '--threads', help='Number of threads to use. [1]', type=int
+    )
     parser.add_argument(
-        '-t', '--threads', help='Number of threads to use. [1]', type=int)
+        '--dups',
+        help='Allow for duplicate query names for more sequence '
+        'coverage across disparate organisms.',
+        action='store_true',
+        default=False,
+    )
     parser.add_argument(
-        '--dups', help='Allow for duplicate query names for more sequence '
-                       'coverage across disparate organisms.',
-        action='store_true', default=False)
+        '--allow_missing', help='Allow for N missing genes per genome. [0]', type=int
+    )
     parser.add_argument(
-        '--allow_missing', help='Allow for N missing genes per genome. [0]',
-        type=int)
+        '--outgroup', help='Name of outgroup file or strain to root on.', type=str
+    )
     parser.add_argument(
-        '--outgroup', help='Name of outgroup file or strain to root on.',
-        type=str)
-    parser.add_argument(
-        '--protect', help='Save files from getting overwritten. By default, '
+        '--protect',
+        help='Save files from getting overwritten. By default, '
         'as input files update, older alignments and trees are deleted.',
-        action='store_true')
+        action='store_true',
+    )
     parser.add_argument(
-        '--checkpoint', help='Name of stage to stop computing on. [none]',
-        type=str, choices=['validate', 'preblast', 'filtering', 'prealign',
-                           'postalign', 'nexus', 'none'])
+        '--checkpoint',
+        help='Name of stage to stop computing on. [none]',
+        type=str,
+        choices=[
+            'validate',
+            'preblast',
+            'filtering',
+            'prealign',
+            'postalign',
+            'nexus',
+            'none',
+        ],
+    )
     parser.add_argument(
-        '--install_deps', help='Install dependencies into given directory. '
-                               '[~/.local/external]',
-        action=InstallDeps)
+        '--install_deps',
+        help='Install dependencies into given directory. ' '[~/.local/external]',
+        action=InstallDeps,
+    )
     parser.add_argument(
-        '--external', help='Path to installed external programs. '
-                           '[~/.local/external]', type=extant_file)
+        '--external',
+        help='Path to installed external programs. ' '[~/.local/external]',
+        type=extant_file,
+    )
+    parser.add_argument('--mafft', help=f'mafft settings [{MAFFT}]', type=str)
+    parser.add_argument('--iqtree', help=f'iqtree2 settings [{IQTREE}]', type=str)
     parser.add_argument(
-        '--mafft', help='mafft settings [--localpair --maxiterate 1000]',
-        type=str)
+        '--debug',
+        help='Turn on debugging messages.',
+        action='store_true',
+        default=False,
+    )
+    parser.add_argument('--version', action='version', version=__version__)
     parser.add_argument(
-        '--iqtree', help='iqtree2 settings '
-        '[-m MFP -B 1000 -alrt 1000 --msub nuclear --merge rclusterf]',
-        type=str)
-    parser.add_argument(
-        '--debug', help='Turn on debugging messages.', action='store_true',
-        default=False)
-    parser.add_argument(
-        '--version', action='version', version=__version__)
-    parser.add_argument(
-        '--quiet', help='Turn off progress messages.', action='store_true',
-        default=False)
+        '--quiet',
+        help='Turn off progress messages.',
+        action='store_true',
+        default=False,
+    )
     args = parser.parse_args()
     args.rundir = config_rundir(args.runid)
     args.configfile = os.path.join(args.rundir, 'config.json')
