@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+# -*- coding: utf-8 -*-
 from __future__ import print_function
 import os.path
 import subprocess
@@ -6,17 +7,22 @@ import logging
 import platform
 import urllib.request
 import shutil
-from pathlib import Path as p
+import re
+import packaging.version as pv
+from pathlib import Path
 from hashlib import md5
-from typing import Dict
+from typing import Dict, Tuple, Union, cast
 from .helper_functions import end_program
-from signal import signal, SIGPIPE, SIGINT, SIG_DFL
-signal(SIGPIPE, SIG_DFL)
-signal(SIGINT, SIG_DFL)
+from ._exceptions import ProgramMismatchError
+from ._versions import (
+    BLASTVERMIN,
+    MAFFTVERMIN,
+    IQTREEVERMIN,
+    BLASTVERCURR,
+    MAFFTVERCURR,
+    IQTREEVERCURR,
+)
 
-BLASTVER = '2.10.1'
-MAFFTVER = '7.471'
-IQTREEVER = '2.1.1'
 EXTERNALDIR = os.path.join(os.path.expanduser('~'), '.local', 'external')
 
 
@@ -35,24 +41,91 @@ def get_external_path(external: str = '') -> str:
     return external
 
 
+def get_program_version(
+    verstr: str, program_name: str
+) -> Tuple[pv.Version, pv.Version]:
+    # Refs from December 2020
+    # mafft: v7.471 (2020/Jul/3)
+    # tblastn: tblastn: 2.10.0+
+    #           Package: blast 2.10.0, build Mar 15 2020 22:36:43
+    # iqtree2: IQ-TREE multicore version 2.1.1 COVID-edition for Linux 64-bit built Aug 22 2020
+    #          Developed by Bui Quang Minh, James Barbetti, Nguyen Lam Tung,
+    #          Olga Chernomor, Heiko Schmidt, Dominik Schrempf, Michael Woodhams.
+    logger = logging.getLogger(__name__)
+    ver: Union[pv.Version, pv.LegacyVersion]
+    refver: Union[pv.Version, pv.LegacyVersion]
+
+    if program_name in ('tblastn', 'blastn', 'makeblastdb'):
+        ver_line = verstr.splitlines()[0]
+        ver = pv.parse(ver_line.split()[1].rstrip('+'))
+        refver = pv.parse(BLASTVERMIN)
+    elif program_name == 'mafft':
+        ver_line = verstr
+        ver = pv.parse(ver_line.split()[0])
+        refver = pv.parse(MAFFTVERMIN)
+    elif program_name == 'iqtree':
+        ver_line = verstr.splitlines()[0]
+        ver_search: str = re.search('version (.+?) ', ver_line).group(1)  # type: ignore
+        ver = pv.parse(ver_search)
+        refver = pv.parse(IQTREEVERMIN)
+    if isinstance(ver, pv.Version):
+        logger.debug(
+            'Program version found: {} -> {}; ref -> {}'.format(
+                program_name, ver, refver
+            )
+        )
+        return (cast(pv.Version, ver), cast(pv.Version, refver))
+    else:
+        raise pv.InvalidVersion(
+            'Unable to determine accurate version for {}: {}'.format(program_name, ver)
+        )
+
+
 def check_program(exe: str, program_name: str, version_flag: str) -> bool:
     logger = logging.getLogger(__name__)
     try:
-        ver: str = subprocess.run([exe, version_flag], check=True,
-                                  stdout=subprocess.PIPE, text=True,
-                                  stderr=subprocess.STDOUT).stdout
+        verstr: str = subprocess.run(
+            [exe, version_flag],
+            check=True,
+            stdout=subprocess.PIPE,
+            text=True,
+            stderr=subprocess.STDOUT,
+        ).stdout
     except FileNotFoundError:
         msg = 'Unable to find {} executable in path or in provided dir ({})'
         logger.error(msg.format(program_name, exe))
         success = False
     except PermissionError:
-        msg = 'Unable to run {} executable in ({}). Make sure it is '\
-            'installed properly and has the correct permissions to run.'
+        msg = (
+            'Unable to run {} executable in ({}). Make sure it is installed properly '
+            'and has the correct permissions to run.'
+        )
         logger.error(msg.format(program_name, exe))
         success = False
     else:
-        logger.debug('{} found: {}'.format(program_name, ver))
-        success = True
+        ver, refver = get_program_version(verstr, program_name)
+        if ver.major == refver.major:
+            if ver >= refver:
+                logger.debug('{} found: {}'.format(program_name, ver))
+                success = True
+            else:
+                raise ProgramMismatchError(
+                    '{} version {} needs to be updated to {} at minimum.'.format(
+                        program_name, ver, refver
+                    )
+                )
+        elif ver.major < refver.major:
+            raise ProgramMismatchError(
+                '{} version {} needs to be updated to {} at minimum.'.format(
+                    program_name, ver, refver
+                )
+            )
+        else:
+            msg = (
+                '{} version {} is newer than tested software. Proceed at your own risk.'
+            )
+            logger.warning(msg.format(program_name, ver))
+
     return success
 
 
@@ -68,21 +141,26 @@ def validate_requirements(external: str = '') -> Dict[str, str]:
     # Set local path to blast executable DIR
     BLASTPATH: str = ''
     externalpath = get_external_path(external)
-    blastcheck = os.path.join(externalpath, 'ncbi-blast-{}+'.format(BLASTVER),
-                              'bin')
+    blastcheck = os.path.join(
+        externalpath, 'ncbi-blast-{}+'.format(BLASTVERCURR), 'bin'
+    )
     if os.path.exists(blastcheck):
         logger.debug('BLAST found in external dir.')
         BLASTPATH = blastcheck
 
     if platform.system() == 'Linux':
         mafftsuffix = 'linux64'
-        iqtreesuffix = os.path.join('{}-Linux'.format(IQTREEVER), 'bin')
+        iqtreesuffix = os.path.join('{}-Linux'.format(IQTREEVERCURR), 'bin')
     elif platform.system() == 'Darwin':
         mafftsuffix = 'mac'
-        iqtreesuffix = os.path.join('{}-MacOSX'.format(IQTREEVER), 'bin')
+        iqtreesuffix = os.path.join('{}-MacOSX'.format(IQTREEVERCURR), 'bin')
     elif platform.system() == 'Windows':
         mafftsuffix = ''
-        iqtreesuffix = os.path.join('{}-Windows'.format(IQTREEVER), 'bin')
+        iqtreesuffix = os.path.join('{}-Windows'.format(IQTREEVERCURR), 'bin')
+    else:
+        logger.warning('Unable to determine os version.')
+        mafftsuffix = ''
+        iqtreesuffix = ''
 
     mafftcheck = os.path.join(externalpath, 'mafft-{}'.format(mafftsuffix))
     iqtreecheck = os.path.join(externalpath, 'iqtree-{}'.format(iqtreesuffix))
@@ -109,38 +187,33 @@ def validate_requirements(external: str = '') -> Dict[str, str]:
 
     logger.debug('Checking tblastn: {}'.format(tblastn))
     if not check_program(tblastn, 'tblastn', '-version'):
-        msg = 'Please add tblastn to your $PATH or supply the path as '\
-            '--external'
+        msg = 'Please add tblastn to your $PATH or supply the path as --external'
         logger.error(msg)
         end_program(78)
 
     logger.debug('Checking blastn: {}'.format(blastn))
     if not check_program(blastn, 'blastn', '-version'):
-        msg = 'Please add blastn to your $PATH or supply the path as '\
-            '--external'
+        msg = 'Please add blastn to your $PATH or supply the path as --external'
         logger.error(msg)
         end_program(78)
 
     logger.debug('Checking makeblastdb: {}'.format(makeblastdb))
     if not check_program(makeblastdb, 'makeblastdb', '-version'):
-        msg = 'Please add makeblastdb to your $PATH or supply the path as '\
-            '--external'
+        msg = 'Please add makeblastdb to your $PATH or supply the path as --external'
         logger.error(msg)
         end_program(78)
 
     # MAFFT testing
     logger.debug('Checking mafft: {}'.format(mafft))
     if not check_program(mafft, 'mafft', '--version'):
-        msg = 'Please add mafft to your $PATH or supply the path as '\
-            '--external'
+        msg = 'Please add mafft to your $PATH or supply the path as --external'
         logger.error(msg)
         end_program(78)
 
     # iqtree testing
     logger.debug('Checking iqtree: {}'.format(iqtree))
     if not check_program(iqtree, 'iqtree', '--version'):
-        msg = 'Please add iqtree to your $PATH or supply the path as '\
-            '--external'
+        msg = 'Please add iqtree to your $PATH or supply the path as --external'
         logger.error(msg)
         end_program(78)
 
@@ -150,7 +223,7 @@ def validate_requirements(external: str = '') -> Dict[str, str]:
     exes['makeblastdb'] = makeblastdb
     exes['mafft'] = mafft
     exes['iqtree'] = iqtree
-    return(exes)
+    return exes
 
 
 def download_file(out_file: str, url: str, program: str) -> None:
@@ -160,8 +233,7 @@ def download_file(out_file: str, url: str, program: str) -> None:
     logger = logging.getLogger(__name__)
     logger.info('Downloading {} from {}.'.format(program, url))
     if not os.path.exists(out_file) or os.path.getsize(out_file) == 0:
-        with urllib.request.urlopen(url) as response, \
-                open(out_file, 'wb') as tarfh:
+        with urllib.request.urlopen(url) as response, open(out_file, 'wb') as tarfh:
             shutil.copyfileobj(response, tarfh)
     else:
         logger.info('{} already downloaded.'.format(program))
@@ -173,21 +245,24 @@ def install_blast(external: str = '') -> None:
     """
     logger = logging.getLogger(__name__)
     externalpath = get_external_path(external)
-    base_url = 'https://ftp.ncbi.nlm.nih.gov/blast/executables/blast+/{}'\
-        .format(BLASTVER)
-    base_file = 'ncbi-blast-{}+-x64-'.format(BLASTVER)
-    if platform.system() == 'Linux':
-        base_out_file = base_file + 'linux.tar.gz'
-    elif platform.system() == 'Darwin':
+    base_url = 'https://ftp.ncbi.nlm.nih.gov/blast/executables/blast+/{}'.format(
+        BLASTVERCURR
+    )
+    base_file = 'ncbi-blast-{}+-x64-'.format(BLASTVERCURR)
+
+    # Assume Linux, update if necessary
+    base_out_file = base_file + 'linux.tar.gz'
+    if platform.system() == 'Darwin':
         base_out_file = base_file + 'macosx.tar.gz'
     elif platform.system() == 'Windows':
         base_out_file = base_file + 'win64.tar.gz'
+
     out_file = os.path.join(externalpath, base_out_file)
     blast_tar_url = '/'.join([base_url, base_out_file])
-    md5_url = '/'.join([base_url, base_out_file + '.md5'])
+    md5_url = '/'.join([base_url, f'{base_out_file}.md5'])
     download_file(out_file, blast_tar_url, 'BLAST')
 
-    blast_md5_digest = md5(p(out_file).read_bytes()).hexdigest()
+    blast_md5_digest = md5(Path(out_file).read_bytes()).hexdigest()
     logger.debug('calculated md5 digest: {}'.format(blast_md5_digest))
 
     download_file(out_file + '.md5', md5_url, 'BLAST md5')
@@ -196,47 +271,51 @@ def install_blast(external: str = '') -> None:
     logger.debug('given md5 digest: {}'.format(md5_hash))
 
     if blast_md5_digest != md5_hash:
-        logger.info('BLAST download was unsuccessful. Delete file and try '
-                    'again.')
+        logger.info('BLAST download was unsuccessful. Delete file and try again.')
         end_program(-1)
-    tblastn = os.path.join(externalpath, 'ncbi-blast-{}+'.format(BLASTVER),
-                           'bin', 'tblastn')
+    tblastn = os.path.join(
+        externalpath, 'ncbi-blast-{}+'.format(BLASTVERCURR), 'bin', 'tblastn'
+    )
     if not os.path.exists(tblastn):
         logger.info('Unpacking BLAST tar.gz file.')
         shutil.unpack_archive(out_file, externalpath, 'gztar')
     logger.info('Checking tblastn version.')
     try:
-        tblastn_ver: bytes = subprocess.check_output([tblastn, '-version'],
-                                                     stderr=subprocess.STDOUT)
+        tblastn_ver: bytes = subprocess.check_output(
+            [tblastn, '-version'], stderr=subprocess.STDOUT
+        )
     except FileNotFoundError:
-        msg = 'Unable to find tblastn executable in path or in provided dir ' \
-              '({})'
+        msg = 'Unable to find tblastn executable in path or in provided dir ({})'
         logger.error(msg.format(tblastn))
         msg = 'Please add tblastn to your $PATH or supply a BLASTPATH in {}'
         logger.error(msg.format(__file__))
         end_program(78)
     else:
         logger.info('tblastn found: {}'.format(tblastn_ver.strip().decode()))
-        logger.info('Add {} to your $PATH to have access to this program '
-                    .format(os.path.dirname(tblastn)) +
-                    'outside of autoMLSA.')
+        logger.info(
+            'Add {} to your $PATH to have access to this program outside of automlsa2'.format(
+                os.path.dirname(tblastn)
+            )
+        )
 
 
 def install_mafft(external: str = '') -> None:
     logger = logging.getLogger(__name__)
 
     base_url = 'https://mafft.cbrc.jp/alignment/software'
-    base_file = 'mafft-{}'.format(MAFFTVER)
+    base_file = 'mafft-{}'.format(MAFFTVERCURR)
     if platform.system() == 'Linux':
         base_out_file = base_file + '-linux.tgz'
-        md5_hash = '641cf15f90fc6634f9b2b2c7cf67e27d'
+        md5_hash = '01ff001442faefdd5d1065973976ea3e'
     elif platform.system() == 'Darwin':
         base_out_file = base_file + '-mac.zip'
-        md5_hash = 'f57cd93ea546e47fbad975aecb58d303'
+        md5_hash = '8206c78533286a4cd6a38821902fbb7a'
     elif platform.system() == 'Windows':
         logger.error('Windows is not currently supported for MAFFT install.')
-        logger.error('Look at install options here: '
-                     'https://mafft.cbrc.jp/alignment/software/windows.html')
+        logger.error(
+            'Look at install options here: '
+            'https://mafft.cbrc.jp/alignment/software/windows.html'
+        )
         return
 
     externalpath = get_external_path(external)
@@ -244,12 +323,11 @@ def install_mafft(external: str = '') -> None:
     tar_url = '/'.join([base_url, base_out_file])
     download_file(out_file, tar_url, 'MAFFT')
 
-    mafft_md5_digest = md5(p(out_file).read_bytes()).hexdigest()
+    mafft_md5_digest = md5(Path(out_file).read_bytes()).hexdigest()
     logger.debug('calculated md5 digest: {}'.format(mafft_md5_digest))
     logger.debug('given md5 digest: {}'.format(md5_hash))
     if mafft_md5_digest != md5_hash:
-        logger.info('MAFFT download was unsuccessful. Delete file and try '
-                    'again.')
+        logger.info('MAFFT download was unsuccessful. Delete file and try again.')
         exit(-1)
     if platform.system() == 'Linux':
         mafft = os.path.join(externalpath, 'mafft-linux64', 'mafft.bat')
@@ -262,92 +340,97 @@ def install_mafft(external: str = '') -> None:
             logger.info('Unpacking MAFFT zip file.')
             shutil.unpack_archive(out_file, externalpath, 'zip')
         logger.debug('Changing permissions.')
-        for root, _, files in \
-                os.walk(os.path.join(externalpath, 'mafft-mac')):
+        for root, _, files in os.walk(os.path.join(externalpath, 'mafft-mac')):
             for exe in files:
                 os.chmod(os.path.join(root, exe), 0o755)
     logger.info('Checking mafft version.')
     try:
-        mafft_ver: bytes = subprocess.check_output([mafft, '--version'],
-                                                   stderr=subprocess.STDOUT)
+        mafft_ver: bytes = subprocess.check_output(
+            [mafft, '--version'], stderr=subprocess.STDOUT
+        )
     except FileNotFoundError:
-        msg = 'Unable to find mafft executable in path or in provided dir ' \
-              '({})'
+        msg = 'Unable to find mafft executable in path or in provided dir ({})'
         logger.error(msg.format(mafft))
         msg = 'Please add mafft to your $PATH or supply a BLASTPATH in {}'
         logger.error(msg.format(__file__))
         end_program(78)
     else:
         logger.info('mafft found: {}'.format(mafft_ver.strip().decode()))
-        logger.info('Add {} to your $PATH to have access to this program '
-                    .format(os.path.dirname(mafft)) +
-                    'ouside of autoMLSA.')
+        logger.info(
+            'Add {} to your $PATH to have access to this program outside of automlsa2'.format(
+                os.path.dirname(mafft)
+            )
+        )
 
 
 def install_iqtree(external: str = '') -> None:
     logger = logging.getLogger(__name__)
 
-    base_url = 'https://github.com/iqtree/iqtree2/releases/download/v{}'\
-        .format(IQTREEVER)
-    base_file = 'iqtree-{}'.format(IQTREEVER)
+    base_url = 'https://github.com/iqtree/iqtree2/releases/download/v{}'.format(
+        IQTREEVERCURR
+    )
+    base_file = 'iqtree-{}'.format(IQTREEVERCURR)
     if platform.system() == 'Linux':
         base_out_file = base_file + '-Linux.tar.gz'
-        md5_hash = '2e2dfeeb2a1e9e123f542b200f18340d'
+        md5_hash = 'fcb2d06c547e597a70c27aff06e63e38'
     elif platform.system() == 'Darwin':
         base_out_file = base_file + '-MacOSX.zip'
-        md5_hash = '45314fba262f200c32c37deceefbf5b1'
+        md5_hash = '889709744247f079e5e79ad8885f78d0'
     elif platform.system() == 'Windows':
         base_out_file = base_file + '-Windows.zip'
-        md5_hash = 'f1a25e89a5c31437125ae0c666d29ee8'
+        md5_hash = '99af9d49bbf4b90bb6a1739a305f5fac'
 
     externalpath = get_external_path(external)
     out_file = os.path.join(externalpath, base_out_file)
     tar_url = '/'.join([base_url, base_out_file])
     download_file(out_file, tar_url, 'iqtree')
 
-    iqtree_md5_digest = md5(p(out_file).read_bytes()).hexdigest()
+    iqtree_md5_digest = md5(Path(out_file).read_bytes()).hexdigest()
     logger.debug('calculated md5 digest: {}'.format(iqtree_md5_digest))
     logger.debug('given md5 digest: {}'.format(md5_hash))
     if iqtree_md5_digest != md5_hash:
-        logger.info('iqtree2 download was unsuccessful. Delete file and try '
-                    'again.')
+        logger.info('iqtree2 download was unsuccessful. Delete file and try again.')
         exit(-1)
     if platform.system() == 'Linux':
-        iqtree = os.path.join(externalpath,
-                              'iqtree-{}-Linux'.format(IQTREEVER),
-                              'bin', 'iqtree2')
+        iqtree = os.path.join(
+            externalpath, 'iqtree-{}-Linux'.format(IQTREEVERCURR), 'bin', 'iqtree2'
+        )
         if not os.path.exists(iqtree):
             logger.info('Unpacking iqtree tar.gz file.')
             shutil.unpack_archive(out_file, externalpath, 'gztar')
     elif platform.system() == 'Darwin':
-        iqtree = os.path.join(externalpath,
-                              'iqtree-{}-MacOSX'.format(IQTREEVER),
-                              'bin', 'iqtree2')
+        iqtree = os.path.join(
+            externalpath, 'iqtree-{}-MacOSX'.format(IQTREEVERCURR), 'bin', 'iqtree2'
+        )
         if not os.path.exists(iqtree):
             logger.info('Unpacking iqtree zip file.')
             shutil.unpack_archive(out_file, externalpath, 'zip')
         os.chmod(iqtree, 0o755)
     elif platform.system() == 'Windows':
-        iqtree = os.path.join(externalpath,
-                              'iqtree-{}-Windows'.format(IQTREEVER),
-                              'bin', 'iqtree2.exe')
+        iqtree = os.path.join(
+            externalpath,
+            'iqtree-{}-Windows'.format(IQTREEVERCURR),
+            'bin',
+            'iqtree2.exe',
+        )
         if not os.path.exists(iqtree):
             logger.info('Unpacking iqtree zip file.')
             shutil.unpack_archive(out_file, externalpath, 'zip')
     logger.info('Checking iqtree version.')
     try:
-        iqtree_ver: bytes = subprocess.check_output([iqtree, '--version'],
-                                                    stderr=subprocess.STDOUT)
+        iqtree_ver: bytes = subprocess.check_output(
+            [iqtree, '--version'], stderr=subprocess.STDOUT
+        )
     except FileNotFoundError:
-        msg = 'Unable to find iqtree2 executable in path or in provided dir ' \
-              '({})'
+        msg = 'Unable to find iqtree2 executable in path or in provided dir ({})'
         logger.error(msg.format(iqtree))
-        msg = 'Please add iqtree2 to your $PATH or supply a full iqtree path '\
-              'in {}'
+        msg = 'Please add iqtree2 to your $PATH or supply a full iqtree path in {}'
         logger.error(msg.format(__file__))
         end_program(78)
     else:
         logger.info('iqtree2 found: {}'.format(iqtree_ver.strip().decode()))
-        logger.info('Add {} to your $PATH to have access to this program '
-                    .format(os.path.dirname(iqtree)) +
-                    'outside of autoMLSA.')
+        logger.info(
+            'Add {} to your $PATH to have access to this program outside of automlsa2'.format(
+                os.path.dirname(iqtree)
+            )
+        )
